package com.howtodoinjava.vaadin.demo;

import com.vaadin.annotations.Theme;
import com.vaadin.annotations.VaadinServletConfiguration;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.Validator.InvalidValueException;
import com.vaadin.data.util.converter.StringToIntegerConverter;
import com.vaadin.data.validator.IntegerRangeValidator;
import com.vaadin.data.validator.StringLengthValidator;
import com.vaadin.event.FieldEvents;
import com.vaadin.event.FieldEvents.TextChangeEvent;
import com.vaadin.event.FieldEvents.TextChangeListener;
import com.vaadin.server.VaadinRequest;
import com.vaadin.server.VaadinServlet;
import com.vaadin.ui.AbstractTextField.TextChangeEventMode;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Notification.Type;
import com.vaadin.ui.TextField;
import com.vaadin.ui.UI;
import com.vaadin.ui.VerticalLayout;
 
 
@Theme("mytheme")
public class MyUI extends UI {
 
    private static final long serialVersionUID = 1387172685749279538L;
 
    @Override
    protected void init(VaadinRequest vaadinRequest) {
        final VerticalLayout layout = new VerticalLayout();
 
        fieldLabelExample(layout);
        fieldSizeExample(layout);
        fieldLimitExample(layout);
        fieldFocusExample(layout);
        fieldValueChangeListenerExample(layout);
        fieldTextChangeListenerExample(layout);
        fieldNullValidationExample(layout);
        fieldNumberValidationExample(layout);
        customCSSExample(layout);
 
        layout.setMargin(true);
        layout.setSpacing(true);
 
        setContent(layout);
    }
     
    private void customCSSExample(VerticalLayout layout) {
        final TextField txtFld = new TextField("Custom Styled Text Box");
        layout.addComponent(txtFld);
         
        layout.addStyleName("darkBorder");
    }
     
    private void fieldNumberValidationExample(VerticalLayout layout) {
         
        final TextField txtFld = new TextField("Please enter only integer value");
        layout.addComponent(txtFld);
         
        //To convert string value to integer before validation
        txtFld.setConverter(new StringToIntegerConverter());
         
        //Add error message and min-max values allowed in field. If no limit than use null
        txtFld.addValidator(new IntegerRangeValidator("The value must be between 0 and 100", 0, 100));
         
        //What if text field is empty - integer will be null in that case, so show blank when null
        txtFld.setNullRepresentation("");
         
        //Add validation hints in UI??
        txtFld.setValidationVisible(true);
         
        txtFld.setTextChangeEventMode(TextChangeEventMode.LAZY);
         
        txtFld.addTextChangeListener(new FieldEvents.TextChangeListener() {
            private static final long serialVersionUID = 1L;
 
            @Override
            public void textChange(FieldEvents.TextChangeEvent event) {
                try {
                    txtFld.setCursorPosition(event.getCursorPosition());
                    txtFld.validate();
                } catch (InvalidValueException e) {
                    //Log error
                }
            }
        });
    }
 
    private void fieldNullValidationExample(VerticalLayout layout) {
 
        final TextField txtFld = new TextField();
        txtFld.setCaption("Empty field validation example");
 
        txtFld.setInputPrompt("Enter your name");
 
        txtFld.setRequired(true);
        txtFld.setRequiredError("Please enter your name to proceed !!");
        txtFld.addValidator(new StringLengthValidator("Must be less than 20 chars", 1, 20, false));
        txtFld.setValidationVisible(true);
 
        txtFld.setTextChangeEventMode(TextChangeEventMode.LAZY);
        txtFld.setNullRepresentation("");
 
        txtFld.addTextChangeListener(new FieldEvents.TextChangeListener() {
            private static final long serialVersionUID = 1L;
 
            @Override
            public void textChange(FieldEvents.TextChangeEvent event) {
                try {
                    txtFld.setValue(event.getText());
 
                    txtFld.setCursorPosition(event.getCursorPosition());
 
                    txtFld.validate();
                } catch (InvalidValueException e) {
                    //Log error
                }
            }
        });
 
        layout.addComponents(txtFld);
    }
 
    private void fieldTextChangeListenerExample(VerticalLayout layout) {
        final TextField txtFld = new TextField();
        txtFld.setCaption("Field text change listner example");
 
        txtFld.focus();
        txtFld.setMaxLength(20);
        txtFld.setInputPrompt("Enter any text");
 
        final Label counter = new Label();
        counter.setValue(txtFld.getValue().length() + " of " + txtFld.getMaxLength());
 
        txtFld.addTextChangeListener(new TextChangeListener() {
            private static final long serialVersionUID = -8043238443902618783L;
 
            public void textChange(TextChangeEvent event) {
                counter.setValue(event.getText().length() + " of "
                        + txtFld.getMaxLength());
            }
        });
 
        txtFld.setTextChangeEventMode(TextChangeEventMode.EAGER);
 
        layout.addComponents(txtFld);
        layout.addComponents(counter);
    }
 
    private void fieldValueChangeListenerExample(VerticalLayout layout) {
        final TextField txtFld = new TextField();
        txtFld.setCaption("Field value change listner example");
 
        txtFld.focus();
        txtFld.setInputPrompt("Enter any text");
 
        txtFld.addValueChangeListener(new Property.ValueChangeListener() {
            private static final long serialVersionUID = 5678485350989070188L;
 
            public void valueChange(ValueChangeEvent event) {
                String value = (String) event.getProperty().getValue();
                // Do something with the value
                Notification.show("Value is: " + value, Type.TRAY_NOTIFICATION);
            }
        });
        // Fire value changes immediately when the field loses focus
        txtFld.setImmediate(true);
 
        layout.addComponents(txtFld);
    }
 
    private void fieldFocusExample(VerticalLayout layout) {
        final TextField txtFld = new TextField();
        txtFld.setCaption("Field focus example");
 
        txtFld.focus();
 
        layout.addComponents(txtFld);
    }
 
    private void fieldLimitExample(VerticalLayout layout) {
        final TextField txtFld = new TextField();
        txtFld.setCaption("Field limit example");
 
        txtFld.setMaxLength(10);
 
        layout.addComponents(txtFld);
    }
 
    private void fieldSizeExample(VerticalLayout layout) {
        final TextField txtFldOne = new TextField();
        txtFldOne.setCaption("Field size example 1");
 
        txtFldOne.setWidth(300, Unit.PIXELS);
        txtFldOne.setHeight(50, Unit.PIXELS);
 
        final TextField txtFldTwo = new TextField();
        txtFldTwo.setCaption("Field size example 2");
 
        txtFldTwo.setWidth("300px");
        txtFldTwo.setHeight("50px");
 
        layout.addComponents(txtFldOne);
        layout.addComponents(txtFldTwo);
    }
 
    private void fieldLabelExample(VerticalLayout layout) {
        final TextField txtFld = new TextField();
        // Set Text Field Label
        txtFld.setCaption("Type your name here:");
        layout.addComponents(txtFld);
    }
 
    @VaadinServletConfiguration(ui = MyUI.class, productionMode = false)
    public static class MyUIServlet extends VaadinServlet {
        private static final long serialVersionUID = -2718268186398688122L;
    }
}